﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VTC6222_C4S_Utility
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            tabControl1.TabPages[0].Text = "System 1";
            tabControl1.TabPages[1].Text = "System 2";
            tabControl1.TabPages[2].Text = "I/O";
            tabControl1.TabPages[3].Text = "Module";
            tabControl1.TabPages[4].Text = "G-Sensor";
            tabControl1.TabPages[5].Text = "PoE";
            tabControl1.TabPages[6].Text = "Event";

            // Main Menu
            MainMenuInit();
            // Page 1 (System 1)
            Page1Init();
            // Page 2 (System 2)
            Page2Init();
            // Page 3 (I/O)
            Page3Init();
            // Page 4 (Module)
            Page4Init();
            // Page 5 (G-Sensor)
            Page5Init();
            // Page 6 (PoE)
            Page6Init();
            // Page 7 (Event)
            Page7Init();

            PageSelectingInit();
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            VTC6222_C4S.Disconnect();
        }

        #region Main Menu
        // ==============================================================================================================
        // Main Menu
        // ==============================================================================================================
        public void MainMenuInit()
        {
            int nMajorVersion = 0;
            int nMinorVersion = 0;
            VTC6222_C4S.Get_DLL_Version(ref nMajorVersion, ref nMinorVersion);

            string strDLLVersion;
            strDLLVersion = "DLL ver : " + nMajorVersion + "." + nMinorVersion;
            textBox_Message.Text = strDLLVersion;

            int nResult = VTC6222_C4S.Connect();
            if (nResult != 1)
            {
                MessageBox.Show("Connect to device failed!!!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //return;
            }
            else
            {
#if DEBUG
                string strHWVersion;
                int nHWVersion = 0;
                nResult = VTC6222_C4S.GetHWVersion(ref nHWVersion);
                if (nResult == 1)
                {
                    int nVer = 65 + nHWVersion;
                    strHWVersion = "PCB : " + Convert.ToString((char)nVer);
                }
                else
                {
                    strHWVersion = "PCB : ?";
                }

                string strDTPVersion;
                int nDTPVersion = 0;
                nResult = VTC6222_C4S.GetDTPVersion(ref nDTPVersion);
                if (nResult == 1)
                {
                    strDTPVersion = "MCU DTP : " + nDTPVersion;
                }
                else
                {
                    strDTPVersion = "MCU DTP : ?";
                }

                string strBLVersion;
                int nBLVersion = 0;
                nResult = VTC6222_C4S.GetBLVersion(ref nBLVersion);
                if (nResult == 1)
                {
                    strBLVersion = "MCU BL : " + nBLVersion;
                }
                else
                {
                    strBLVersion = "MCU BL : ?";
                }

                textBox_Message.Text = strDLLVersion + ", " + strHWVersion + ", " + strDTPVersion + ", " + strBLVersion;
#endif
            }
        }

        private void button_LoadDefault_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetLoadDefault();
            if (nResult == 1)
            {
                textBox_Message.Text = "Load Default Success.";
                MessageBox.Show("Loading factory default settings successfully.", this.Text);

                ReadingAfterLoadDefault();
            }
            else
            {
                textBox_Message.Text = "Load Default Failed!!!";
                MessageBox.Show("Loading factory default settings failed!!!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void button_Save_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetFlashUpdate();
            if (nResult == 1)
            {
                textBox_Message.Text = "Update Flash Success.";

                string strMessage;
                strMessage = "The following settings have been saved :\n\n" +
                             "1. Startup/Shutdown voltage\n" +
                             "2. Power On/Off Delay Timer\n" +
                             "3. WWAN\n" +
                             "4. Wake On LAN\n" +
                             "5. RTC Wake Up Timer";
                MessageBox.Show(strMessage, this.Text);
            }
            else
            {
                textBox_Message.Text = "Update Flash Failed!!!";
                MessageBox.Show("Failed to save settings!!!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
        #endregion

        #region Page 1 (System 1)
        // ==============================================================================================================
        // Page 1 (System 1)
        // ==============================================================================================================
        public void Page1Init()
        {
            comboBox_StartupShutdown.Items.Clear();
            comboBox_StartupShutdown.Items.Add("11.5V / 10.5V       23.0V / 21.0V");
            comboBox_StartupShutdown.Items.Add("12.0V / 11.0V       24.0V / 22.0V");
            comboBox_StartupShutdown.Items.Add("12.5V / 11.0V       25.0V / 22.0V");
            comboBox_StartupShutdown.Items.Add("12.5V / 11.5V       25.0V / 23.0V");
            comboBox_StartupShutdown.SelectedIndex = 0;

            // BIOS Ver
            {
                // The "Get_BIOS_Version" call "CoInitializeEx(NULL, COINIT_MULTITHREADED)" to access SMBIOS,
                // so C# must use Thread to call "Get_BIOS_Version".

                Thread_Get_BIOS_Version = new System.Threading.Thread(Thread_Get_BIOS_Version_Function);
                Thread_Get_BIOS_Version.Priority = System.Threading.ThreadPriority.Highest;
                Thread_Get_BIOS_Version.Start();
                while (!Thread_Get_BIOS_Version.IsAlive) ;
            }
        }

        private System.Threading.Thread Thread_Get_BIOS_Version = null;
        public void Thread_Get_BIOS_Version_Function()
        {
            // BIOS Ver
#if USING_WCHAR
            string strBIOS_Version = new string('\0', 32);
            bool bResult = VTC6222_C4S.Get_BIOS_Version_W(strBIOS_Version, 32);
#else
            StringBuilder strbBIOS_Version = new StringBuilder(32);
            bool bResult = VTC6222_C4S.Get_BIOS_Version(strbBIOS_Version, 32);
            string strBIOS_Version = strbBIOS_Version.ToString();
#endif
            if (bResult)
            {
                SetTextBox_BIOS_Version(strBIOS_Version);
            }
            else
            {
                SetTextBox_BIOS_Version("unknown");
            }
        }

        delegate void SetTextBox_BIOS_Version_Callback(string text);
        private void SetTextBox_BIOS_Version(string text)
        {
            if (this.textBox_BIOS_Version.InvokeRequired)
            {
                SetTextBox_BIOS_Version_Callback d = new SetTextBox_BIOS_Version_Callback(SetTextBox_BIOS_Version);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.textBox_BIOS_Version.Text = text;
            }
        }

        public void fnGetMCUVersion()
        {
            int nMCUVersion = 0;
            int nResult = VTC6222_C4S.GetMCUVersion(ref nMCUVersion);
            if (nResult == 1)
            {
                textBox_MCU_Version.Text = "VT642R" + nMCUVersion.ToString("D2");

            }
            else
            {
                textBox_MCU_Version.Text = "?";
            }
        }

        public void fnGetIgnitionStatus()
        {
            int nIgnitionStatus = 0;
            int nResult = VTC6222_C4S.GetIgnitionStatus(ref nIgnitionStatus);
            if (nResult == 1)
            {
                switch (nIgnitionStatus)
                {
                    case 0: textBox_Ignition.Text = "OFF"; break;
                    case 1: textBox_Ignition.Text = "ON"; break;
                    default: textBox_Ignition.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_Ignition.Text = "?";
            }
        }

        public void fnGetSupplyVoltage()
        {
            int nSupplyVoltage = 0;
            int nResult = VTC6222_C4S.GetSupplyVoltage(ref nSupplyVoltage);
            if (nResult == 1)
            {
                double fVoltage = (double)nSupplyVoltage / 1000;
                textBox_Input_Voltage.Text = fVoltage.ToString("F01") + " V";
            }
            else
            {
                textBox_Input_Voltage.Text = "?";
            }
        }

        public void fnGetPowerType()
        {
            int nPowerType = 0;
            int nResult = VTC6222_C4S.GetPowerType(ref nPowerType);
            if (nResult == 1)
            {
                switch (nPowerType)
                {
                    case 0: textBox_Power_Type.Text = "Reserved(9~48V)"; break;
                    case 1: textBox_Power_Type.Text = "9~48V"; break;
                    case 2: textBox_Power_Type.Text = "24V"; break;
                    case 3: textBox_Power_Type.Text = "12V"; break;
                    default: textBox_Power_Type.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_Power_Type.Text = "?";
            }
        }

        public void fnGetStartupShutdownOption()
        {
            int nStartupShutdownOption = 0;
            int nResult = VTC6222_C4S.GetStartupShutdownOption(ref nStartupShutdownOption);
            if (nResult == 1)
            {
                switch (nStartupShutdownOption)
                {
                    case 0: textBox_StartupShutdown.Text = "11.5V / 10.5V       23.0V / 21.0V"; break;
                    case 1: textBox_StartupShutdown.Text = "12.0V / 11.0V       24.0V / 22.0V"; break;
                    case 2: textBox_StartupShutdown.Text = "12.5V / 11.0V       25.0V / 22.0V"; break;
                    case 3: textBox_StartupShutdown.Text = "12.5V / 11.5V       25.0V / 23.0V"; break;
                    default: textBox_StartupShutdown.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_StartupShutdown.Text = "?";
            }
        }

        private void button_Get_Power_Type_Click(object sender, EventArgs e)
        {
            fnGetPowerType();
        }

        private void button_Set_StartupShutdown_Click(object sender, EventArgs e)
        {
            int nOption = comboBox_StartupShutdown.SelectedIndex;
            int nResult = VTC6222_C4S.SetStartupShutdownOption(nOption);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Startup Shutdown Option Success.";

                fnGetStartupShutdownOption();
            }
            else
            {
                textBox_Message.Text = "Set Startup Shutdown Option Failed!!!";
            }
        }
        #endregion

        #region Page 2 (System 2)
        // ==============================================================================================================
        // Page 2 (System 2)
        // ==============================================================================================================
        public void Page2Init()
        {
            comboBox_PowerOffDelay.Items.Clear();
            comboBox_PowerOffDelay.Items.Add("Disable");
            comboBox_PowerOffDelay.Items.Add("Enable");
            comboBox_PowerOffDelay.SelectedIndex = 0;

            comboBox_DelayOffTime.Items.Clear();
            comboBox_DelayOffTime.Items.Add("20 Sec");
            comboBox_DelayOffTime.Items.Add("1 Min");
            comboBox_DelayOffTime.Items.Add("5 Mins");
            comboBox_DelayOffTime.Items.Add("10 Mins");
            comboBox_DelayOffTime.Items.Add("30 Mins");
            comboBox_DelayOffTime.Items.Add("1 Hour");
            comboBox_DelayOffTime.Items.Add("6 Hours");
            comboBox_DelayOffTime.Items.Add("18 Hours");
            comboBox_DelayOffTime.SelectedIndex = 0;

            comboBox_PowerOnDelay.Items.Clear();
            comboBox_PowerOnDelay.Items.Add("Disable");
            comboBox_PowerOnDelay.Items.Add("Enable");
            comboBox_PowerOnDelay.SelectedIndex = 0;

            comboBox_DelayOnTime.Items.Clear();
            comboBox_DelayOnTime.Items.Add("10 Sec");
            comboBox_DelayOnTime.Items.Add("30 Sec");
            comboBox_DelayOnTime.Items.Add("1Min");
            comboBox_DelayOnTime.Items.Add("5 Mins");
            comboBox_DelayOnTime.Items.Add("10 Mins");
            comboBox_DelayOnTime.Items.Add("15 Mins");
            comboBox_DelayOnTime.Items.Add("30 Mins");
            comboBox_DelayOnTime.Items.Add("1 Hour");
            comboBox_DelayOnTime.SelectedIndex = 0;

            comboBox_RTCWakeUp.Items.Clear();
            comboBox_RTCWakeUp.Items.Add("Disable");
            comboBox_RTCWakeUp.Items.Add("Enable");
            comboBox_RTCWakeUp.SelectedIndex = 0;

            comboBox_RTCHour.Items.Clear();
            for (int i = 0; i <= 23; i++)
            {
                comboBox_RTCHour.Items.Add(i.ToString());
            }
            comboBox_RTCHour.SelectedIndex = 0;

            comboBox_RTCMin.Items.Clear();
            for (int i = 0; i <= 59; i++)
            {
                comboBox_RTCMin.Items.Add(i.ToString());
            }
            comboBox_RTCMin.SelectedIndex = 0;

            comboBox_RTCSec.Items.Clear();
            for (int i = 0; i <= 59; i++)
            {
                comboBox_RTCSec.Items.Add(i.ToString());
            }
            comboBox_RTCSec.SelectedIndex = 0;

            comboBox_WDT.Items.Clear();
            comboBox_WDT.Items.Add("Disable");
            comboBox_WDT.Items.Add("Enable");
            comboBox_WDT.SelectedIndex = 0;

            comboBox_WDT_Interval.Items.Clear();
            for (int i = 3; i <= 255; i++)
            {
                comboBox_WDT_Interval.Items.Add(i.ToString());
            }
            comboBox_WDT_Interval.SelectedIndex = 0;
        }

        public void fnGetDelayTimeOption()
        {
            int nPowerOff = 0;
            int nPowerOn = 0;
            int nDelayOffEnable = 0;
            int nDelayOnEnable = 0;

            int nResult = VTC6222_C4S.GetDelayTimeOption(ref nPowerOff, ref nPowerOn, ref nDelayOffEnable, ref nDelayOnEnable);
            if (nResult == 1)
            {
                switch (nPowerOff)
                {
                    case 0: textBox_DelayOffTime.Text = "20 sec"; break;
                    case 1: textBox_DelayOffTime.Text = "1 Min"; break;
                    case 2: textBox_DelayOffTime.Text = "5 Mins"; break;
                    case 3: textBox_DelayOffTime.Text = "10 Mins"; break;
                    case 4: textBox_DelayOffTime.Text = "30 Mins"; break;
                    case 5: textBox_DelayOffTime.Text = "1 Hour"; break;
                    case 6: textBox_DelayOffTime.Text = "6 Hours"; break;
                    case 7: textBox_DelayOffTime.Text = "18 Hours"; break;
                    default: textBox_DelayOffTime.Text = "Undefined"; break;
                }

                switch (nPowerOn)
                {
                    case 0: textBox_DelayOnTime.Text = "10 sec"; break;
                    case 1: textBox_DelayOnTime.Text = "30 sec"; break;
                    case 2: textBox_DelayOnTime.Text = "1 Min"; break;
                    case 3: textBox_DelayOnTime.Text = "5 Mins"; break;
                    case 4: textBox_DelayOnTime.Text = "10 Mins"; break;
                    case 5: textBox_DelayOnTime.Text = "15 Mins"; break;
                    case 6: textBox_DelayOnTime.Text = "30 Mins"; break;
                    case 7: textBox_DelayOnTime.Text = "1 Hour"; break;
                    default: textBox_DelayOnTime.Text = "Undefined"; break;
                }

                switch (nDelayOffEnable)
                {
                    case 0: textBox_PowerOffDelay.Text = "Disable"; break;
                    case 1: textBox_PowerOffDelay.Text = "Enable"; break;
                    default: textBox_PowerOffDelay.Text = "Undefined"; break;
                }

                switch (nDelayOnEnable)
                {
                    case 0: textBox_PowerOnDelay.Text = "Disable"; break;
                    case 1: textBox_PowerOnDelay.Text = "Enable"; break;
                    default: textBox_PowerOnDelay.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_DelayOffTime.Text = "?";
                textBox_DelayOnTime.Text = "?";
                textBox_PowerOffDelay.Text = "?";
                textBox_PowerOnDelay.Text = "?";
            }
        }

        public void fnGetRTCStatus()
        {
            int nRTCStatus = 0;
            int nResult = VTC6222_C4S.GetRTCStatus(ref nRTCStatus);
            if (nResult == 1)
            {
                switch (nRTCStatus)
                {
                    case 0: textBox_RTCWakeUp.Text = "Disable"; break;
                    case 1: textBox_RTCWakeUp.Text = "Enable"; break;
                    default: textBox_RTCWakeUp.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_RTCWakeUp.Text = "?";
            }
        }

        public void fnGetAlarmTimer()
        {
            int nHour = 0;
            int nMin = 0;
            int nSec = 0;
            int nResult = VTC6222_C4S.GetAlarmTimer(ref nHour, ref nMin, ref nSec);
            if (nResult == 1)
            {
                textBox_RTCHour.Text = nHour.ToString();
                textBox_RTCMin.Text = nMin.ToString();
                textBox_RTCSec.Text = nSec.ToString();
            }
            else
            {
                textBox_RTCHour.Text = "?";
                textBox_RTCMin.Text = "?";
                textBox_RTCSec.Text = "?";
            }
        }

        public void fnGetWatchdogConfig()
        {
            int nWDT_Enable = 0;
            int nTimeInterval = 0;
            int nResult = VTC6222_C4S.GetWatchdogConfig(ref nWDT_Enable, ref nTimeInterval);
            if (nResult == 1)
            {
                switch (nWDT_Enable)
                {
                    case 0: textBox_WDT.Text = "Disable"; break;
                    case 1: textBox_WDT.Text = "Enable"; break;
                    default: textBox_WDT.Text = "Undefined"; break;
                }

                textBox_WDT_Interval.Text = nTimeInterval.ToString() + " Sec";
            }
            else
            {
                textBox_WDT.Text = "?";
                textBox_WDT_Interval.Text = "?";
            }
        }

        private void button_Set_PowerDelayTimer_Click(object sender, EventArgs e)
        {
            int nPowerOffDelayTime = comboBox_DelayOffTime.SelectedIndex;
            int nPowerOffDelayTime_Enable = comboBox_PowerOffDelay.SelectedIndex;
            int nPowerOnDelayTime = comboBox_DelayOnTime.SelectedIndex;
            int nPowerOnDelayTime_Enable = comboBox_PowerOnDelay.SelectedIndex;
            int nResult = VTC6222_C4S.SetDelayTimeOption(nPowerOffDelayTime, nPowerOffDelayTime_Enable, nPowerOnDelayTime, nPowerOnDelayTime_Enable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Delay Time Option Success.";

                fnGetDelayTimeOption();
            }
            else
            {
                textBox_Message.Text = "Set Delay Time Option Failed!!!";
            }
        }

        private void button_Set_RTCWakeUp_Click(object sender, EventArgs e)
        {
            int nEnable = comboBox_RTCWakeUp.SelectedIndex;
            int nResult = VTC6222_C4S.SetRTCControl(nEnable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set RTC Control Success.";

                fnGetRTCStatus();
            }
            else
            {
                textBox_Message.Text = "Set RTC Control Failed!!!";
            }
        }

        private void button_Set_RTC_Timer_Click(object sender, EventArgs e)
        {
            DateTime LocalTime = DateTime.Now;
            int nResult = VTC6222_C4S.SetRTCTimer(LocalTime.Hour, LocalTime.Minute, LocalTime.Second);
            if (nResult == 1)
            {
                int nHour = comboBox_RTCHour.SelectedIndex;
                int nMin = comboBox_RTCMin.SelectedIndex;
                int nSec = comboBox_RTCSec.SelectedIndex;
                nResult = VTC6222_C4S.SetAlarmTimer(nHour, nMin, nSec);
                if (nResult == 1)
                {
                    textBox_Message.Text = "Set Alarm Timer Success.";

                    fnGetAlarmTimer();
                }
                else
                {
                    textBox_Message.Text = "Set Alarm Timer Failed!!!";
                }
            }
            else
            {
                textBox_Message.Text = "Set RTC Timer Failed!!!";
            }
        }

        private void button_Set_WDT_Click(object sender, EventArgs e)
        {
            int nWDT_Enable = comboBox_WDT.SelectedIndex;
            int nResult = VTC6222_C4S.SetWatchdogControl(nWDT_Enable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Watchdog Control success.";

                fnGetWatchdogConfig();
            }
            else
            {
                textBox_Message.Text = "Set Watchdog Control failed!!!";
            }
        }

        private void button_Set_WDT_Interval_Click(object sender, EventArgs e)
        {
            int nTimeInterval = comboBox_WDT_Interval.SelectedIndex + 3;
            int nResult = VTC6222_C4S.SetWatchdogTimer(nTimeInterval);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Watchdog Timer success.";

                fnGetWatchdogConfig();
            }
            else
            {
                textBox_Message.Text = "Set Watchdog Timer failed!!!";
            }
        }
        #endregion

        #region Page 3 (I/O)
        // ==============================================================================================================
        // Page 3 (I/O)
        // ==============================================================================================================
        public void Page3Init()
        {
            comboBox_GPO1.Items.Clear();
            comboBox_GPO1.Items.Add("Low");
            comboBox_GPO1.Items.Add("High");
            comboBox_GPO1.SelectedIndex = 1;

            comboBox_GPO2.Items.Clear();
            comboBox_GPO2.Items.Add("Low");
            comboBox_GPO2.Items.Add("High");
            comboBox_GPO2.SelectedIndex = 1;

            comboBox_GPO3.Items.Clear();
            comboBox_GPO3.Items.Add("Low");
            comboBox_GPO3.Items.Add("High");
            comboBox_GPO3.SelectedIndex = 1;

            comboBox_GPO4.Items.Clear();
            comboBox_GPO4.Items.Add("Low");
            comboBox_GPO4.Items.Add("High");
            comboBox_GPO4.SelectedIndex = 1;

            comboBox_WakeOnLan.Items.Clear();
            comboBox_WakeOnLan.Items.Add("Disable");
            comboBox_WakeOnLan.Items.Add("Enable");
            comboBox_WakeOnLan.SelectedIndex = 0;

            comboBox_External12V.Items.Clear();
            comboBox_External12V.Items.Add("Disable");
            comboBox_External12V.Items.Add("Enable");
            comboBox_External12V.SelectedIndex = 0;

            comboBox_LED1.Items.Clear();
            comboBox_LED1.Items.Add("OFF");
            comboBox_LED1.Items.Add("ON");
            comboBox_LED1.SelectedIndex = 0;

            comboBox_LED2.Items.Clear();
            comboBox_LED2.Items.Add("OFF");
            comboBox_LED2.Items.Add("ON");
            comboBox_LED2.SelectedIndex = 0;

            comboBox_LED3.Items.Clear();
            comboBox_LED3.Items.Add("OFF");
            comboBox_LED3.Items.Add("ON");
            comboBox_LED3.SelectedIndex = 0;

            comboBox_LED4.Items.Clear();
            comboBox_LED4.Items.Add("OFF");
            comboBox_LED4.Items.Add("ON");
            comboBox_LED4.SelectedIndex = 0;
        }

        public void fnGetGPOStatus()
        {
            int nGPO1 = 0;
            int nGPO2 = 0;
            int nGPO3 = 0;
            int nGPO4 = 0;
            int nResult = VTC6222_C4S.GetGPOStatus(ref nGPO1, ref nGPO2, ref nGPO3, ref nGPO4);
            if (nResult == 1)
            {
                switch (nGPO1)
                {
                    case 0: textBox_GPO1.Text = "Low"; break;
                    case 1: textBox_GPO1.Text = "High"; break;
                    default: textBox_GPO1.Text = "Undefined"; break;
                }

                switch (nGPO2)
                {
                    case 0: textBox_GPO2.Text = "Low"; break;
                    case 1: textBox_GPO2.Text = "High"; break;
                    default: textBox_GPO2.Text = "Undefined"; break;
                }

                switch (nGPO3)
                {
                    case 0: textBox_GPO3.Text = "Low"; break;
                    case 1: textBox_GPO3.Text = "High"; break;
                    default: textBox_GPO3.Text = "Undefined"; break;
                }

                switch (nGPO4)
                {
                    case 0: textBox_GPO4.Text = "Low"; break;
                    case 1: textBox_GPO4.Text = "High"; break;
                    default: textBox_GPO4.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_GPO1.Text = "?";
                textBox_GPO2.Text = "?";
                textBox_GPO3.Text = "?";
                textBox_GPO4.Text = "?";
            }
        }

        public void fnGetGPIStatus()
        {
            int nGPI1 = 0;
            int nGPI2 = 0;
            int nGPI3 = 0;
            int nGPI4 = 0;
            int nResult = VTC6222_C4S.GetGPIStatus(ref nGPI1, ref nGPI2, ref nGPI3, ref nGPI4);
            if (nResult == 1)
            {
                switch (nGPI1)
                {
                    case 0: textBox_GPI1.Text = "Low"; break;
                    case 1: textBox_GPI1.Text = "High"; break;
                    default: textBox_GPI1.Text = "Undefined"; break;
                }

                switch (nGPI2)
                {
                    case 0: textBox_GPI2.Text = "Low"; break;
                    case 1: textBox_GPI2.Text = "High"; break;
                    default: textBox_GPI2.Text = "Undefined"; break;
                }

                switch (nGPI3)
                {
                    case 0: textBox_GPI3.Text = "Low"; break;
                    case 1: textBox_GPI3.Text = "High"; break;
                    default: textBox_GPI3.Text = "Undefined"; break;
                }

                switch (nGPI4)
                {
                    case 0: textBox_GPI4.Text = "Low"; break;
                    case 1: textBox_GPI4.Text = "High"; break;
                    default: textBox_GPI4.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_GPI1.Text = "?";
                textBox_GPI2.Text = "?";
                textBox_GPI3.Text = "?";
                textBox_GPI4.Text = "?";
            }
        }

        public void fnGetWakeOnLanStatus()
        {
            int nWakeOnLanStatus = 0;
            int nResult = VTC6222_C4S.GetWakeOnLanStatus(ref nWakeOnLanStatus);
            if (nResult == 1)
            {
                switch (nWakeOnLanStatus)
                {
                    case 0: textBox_WakeOnLan.Text = "Disable"; break;
                    case 1: textBox_WakeOnLan.Text = "Enable"; break;
                    default: textBox_WakeOnLan.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_WakeOnLan.Text = "?";
            }
        }

        public void fnGetExternal12VStatus()
        {
            int nExternal12VStatus = 0;
            int nResult = VTC6222_C4S.GetExternal12VStatus(ref nExternal12VStatus);
            if (nResult == 1)
            {
                switch (nExternal12VStatus)
                {
                    case 0: textBox_External12V.Text = "Disable"; break;
                    case 1: textBox_External12V.Text = "Enable"; break;
                    default: textBox_External12V.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_External12V.Text = "?";
            }
        }

        public void fnGetProgramLEDStatus()
        {
            int nLED_1 = 0;
            int nLED_2 = 0;
            int nLED_3 = 0;
            int nLED_4 = 0;

            int nResult = VTC6222_C4S.GetProgramLEDStatus(ref nLED_1, ref nLED_2, ref nLED_3, ref nLED_4);
            if (nResult == 1)
            {
                switch (nLED_1)
                {
                    case 0: textBox_LED1.Text = "OFF"; break;
                    case 1: textBox_LED1.Text = "ON"; break;
                    default: textBox_LED1.Text = "Undefined"; break;
                }

                switch (nLED_2)
                {
                    case 0: textBox_LED2.Text = "OFF"; break;
                    case 1: textBox_LED2.Text = "ON"; break;
                    default: textBox_LED2.Text = "Undefined"; break;
                }

                switch (nLED_3)
                {
                    case 0: textBox_LED3.Text = "OFF"; break;
                    case 1: textBox_LED3.Text = "ON"; break;
                    default: textBox_LED3.Text = "Undefined"; break;
                }

                switch (nLED_4)
                {
                    case 0: textBox_LED4.Text = "OFF"; break;
                    case 1: textBox_LED4.Text = "ON"; break;
                    default: textBox_LED4.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_LED1.Text = "?";
                textBox_LED2.Text = "?";
                textBox_LED3.Text = "?";
                textBox_LED4.Text = "?";
            }
        }

        private void button_Set_GPO_Click(object sender, EventArgs e)
        {
            int nGPO1 = comboBox_GPO1.SelectedIndex;
            int nGPO2 = comboBox_GPO2.SelectedIndex;
            int nGPO3 = comboBox_GPO3.SelectedIndex;
            int nGPO4 = comboBox_GPO4.SelectedIndex;
            int nResult = VTC6222_C4S.SetGPOControl(nGPO1, nGPO2, nGPO3, nGPO4);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set GPO success.";

                fnGetGPOStatus();
            }
            else
            {
                textBox_Message.Text = "Set GPO failed!!!";
            }
        }

        private void button_Get_GPI_Click(object sender, EventArgs e)
        {
            fnGetGPIStatus();
        }

        private void button_Set_WakeOnLan_Click(object sender, EventArgs e)
        {
            int nEnable = comboBox_WakeOnLan.SelectedIndex;
            int nResult = VTC6222_C4S.SetWakeOnLanControl(nEnable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Wake On Lan Success.";

                fnGetWakeOnLanStatus();
            }
            else
            {
                textBox_Message.Text = "Set Wake On Lan Failed!!!";
            }
        }

        private void button_Set_External12V_Click(object sender, EventArgs e)
        {
            int nEnable = comboBox_External12V.SelectedIndex;
            int nResult = VTC6222_C4S.SetExternal12VControl(nEnable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set External 12V Control Success.";

                fnGetExternal12VStatus();
            }
            else
            {
                textBox_Message.Text = "Set External 12V Control Failed!!!";
            }
        }

        private void button_Set_LED_Click(object sender, EventArgs e)
        {
            int nLED1 = comboBox_LED1.SelectedIndex;
            int nLED2 = comboBox_LED2.SelectedIndex;
            int nLED3 = comboBox_LED3.SelectedIndex;
            int nLED4 = comboBox_LED4.SelectedIndex;

            int nResult = VTC6222_C4S.SetProgramLEDControl(nLED1, nLED2, nLED3, nLED4);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Program LED Control Success.";

                fnGetProgramLEDStatus();
            }
            else
            {
                textBox_Message.Text = "Set Program LED Control Failed!!!";
            }
        }

        #endregion

        #region Page 4 (Module)
        // ==============================================================================================================
        // Page 4 (Module)
        // ==============================================================================================================
        public void Page4Init()
        {
            comboBox_WWAN1.Items.Clear();
            comboBox_WWAN1.Items.Add("Disable");
            comboBox_WWAN1.Items.Add("Enable");
            comboBox_WWAN1.SelectedIndex = 1;

            comboBox_WWAN1_Wakeup.Items.Clear();
            comboBox_WWAN1_Wakeup.Items.Add("Disable");
            comboBox_WWAN1_Wakeup.Items.Add("Enable");
            comboBox_WWAN1_Wakeup.SelectedIndex = 0;

            comboBox_WiFi1.Items.Clear();
            comboBox_WiFi1.Items.Add("Disable");
            comboBox_WiFi1.Items.Add("Enable");
            comboBox_WiFi1.SelectedIndex = 1;

            comboBox_WiFi2.Items.Clear();
            comboBox_WiFi2.Items.Add("Disable");
            comboBox_WiFi2.Items.Add("Enable");
            comboBox_WiFi2.SelectedIndex = 1;

            comboBox_BT1.Items.Clear();
            comboBox_BT1.Items.Add("Disable");
            comboBox_BT1.Items.Add("Enable");
            comboBox_BT1.SelectedIndex = 1;

            comboBox_BT2.Items.Clear();
            comboBox_BT2.Items.Add("Disable");
            comboBox_BT2.Items.Add("Enable");
            comboBox_BT2.SelectedIndex = 1;

            comboBox_GPS.Items.Clear();
            comboBox_GPS.Items.Add("Disable");
            comboBox_GPS.Items.Add("Enable");
            comboBox_GPS.SelectedIndex = 1;

            comboBox_SIM1.Items.Clear();
            comboBox_SIM1.Items.Add("SIM Card 1 (CN17)");
            comboBox_SIM1.Items.Add("SIM Card 2 (CN13)");
            comboBox_SIM1.SelectedIndex = 0;
        }

        public void fnGetWWANStatus()
        {
            int nWWAN_Enable = 0;
            int nWWAN_Wakeup_Enable = 0;

            int nResult = VTC6222_C4S.GetWWANStatus(ref nWWAN_Enable, ref nWWAN_Wakeup_Enable);
            if (nResult == 1)
            {
                switch (nWWAN_Enable)
                {
                    case 0: textBox_WWAN1.Text = "Disable"; break;
                    case 1: textBox_WWAN1.Text = "Enable"; break;
                    default: textBox_WWAN1.Text = "Undefined"; break;
                }

                switch (nWWAN_Wakeup_Enable)
                {
                    case 0: textBox_WWAN1_Wakeup.Text = "Disable"; break;
                    case 1: textBox_WWAN1_Wakeup.Text = "Enable"; break;
                    default: textBox_WWAN1_Wakeup.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_WWAN1.Text = "?";
                textBox_WWAN1_Wakeup.Text = "?";
            }
        }

        public void fnGetWiFiStatus()
        {
            int nWiFi_En1 = 0;
            int nWiFi_En2 = 0;
            int nWiFi_Pw1 = 1; // Power on/off is used for USB module only.
            int nWiFi_Pw2 = 1; // Power on/off is used for USB module only.

            int nResult = VTC6222_C4S.GetWiFiStatus(ref nWiFi_En1, ref nWiFi_En2, ref nWiFi_Pw1, ref nWiFi_Pw2);
            if (nResult == 1)
            {
                switch (nWiFi_En1)
                {
                    case 0: textBox_WiFi1.Text = "Disable"; break;
                    case 1: textBox_WiFi1.Text = "Enable"; break;
                    default: textBox_WiFi1.Text = "Undefined"; break;
                }

                switch (nWiFi_En2)
                {
                    case 0: textBox_WiFi2.Text = "Disable"; break;
                    case 1: textBox_WiFi2.Text = "Enable"; break;
                    default: textBox_WiFi2.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_WiFi1.Text = "?";
                textBox_WiFi2.Text = "?";
            }
        }

        public void fnGetBTStatus()
        {
            int nBTStatus1 = 0;
            int nBTStatus2 = 0;

            int nResult = VTC6222_C4S.GetBTStatus(ref nBTStatus1, ref nBTStatus2);
            if (nResult == 1)
            {
                switch (nBTStatus1)
                {
                    case 0: textBox_BT1.Text = "Disable"; break;
                    case 1: textBox_BT1.Text = "Enable"; break;
                    default: textBox_BT1.Text = "Undefined"; break;
                }

                switch (nBTStatus2)
                {
                    case 0: textBox_BT2.Text = "Disable"; break;
                    case 1: textBox_BT2.Text = "Enable"; break;
                    default: textBox_BT2.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_BT1.Text = "?";
                textBox_BT2.Text = "?";
            }
        }

        public void fnGetGPSStatus()
        {
            int nGPSStatus = 0;
            int nResult = VTC6222_C4S.GetGPSStatus(ref nGPSStatus);
            if (nResult == 1)
            {
                switch (nGPSStatus)
                {
                    case 0: textBox_GPS.Text = "Disable"; break;
                    case 1: textBox_GPS.Text = "Enable"; break;
                    default: textBox_GPS.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_GPS.Text = "?";
            }
        }

        public void fnGetSIMCardStatus()
        {
            int nSIMCard1 = 0;

            int nResult = VTC6222_C4S.GetSIMCardStatus(ref nSIMCard1);
            if (nResult == 1)
            {
                switch (nSIMCard1)
                {
                    case 0: textBox_SIM1.Text = "1"; break;
                    case 1: textBox_SIM1.Text = "2"; break;
                    default: textBox_SIM1.Text = "Undefined"; break;
                }
            }
            else
            {
                textBox_SIM1.Text = "?";
            }
        }

        private void button_Set_WWAN_Click(object sender, EventArgs e)
        {
            int nWWAN_Enable = comboBox_WWAN1.SelectedIndex;
            int nWWAN_Wakeup_Enable = comboBox_WWAN1_Wakeup.SelectedIndex;

            int nResult = VTC6222_C4S.SetWWANControl(nWWAN_Enable, nWWAN_Wakeup_Enable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set WWAN Control success.";

                fnGetWWANStatus();
            }
            else
            {
                textBox_Message.Text = "Set WWAN Control failed!!!";
            }
        }

        private void button_Set_WiFi_Click(object sender, EventArgs e)
        {
            int nWiFi_En1 = comboBox_WiFi1.SelectedIndex;
            int nWiFi_En2 = comboBox_WiFi2.SelectedIndex;
            int nWiFi_Pw1 = 1; // Power on/off is used for USB module only.
            int nWiFi_Pw2 = 1; // Power on/off is used for USB module only.

            int nResult = VTC6222_C4S.SetWiFiControl(nWiFi_En1, nWiFi_En2, nWiFi_Pw1, nWiFi_Pw2);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set WiFi Control success.";

                fnGetWiFiStatus();
            }
            else
            {
                textBox_Message.Text = "Set WiFi Control failed!!!";
            }
        }

        private void button_Set_BT_Click(object sender, EventArgs e)
        {
            int nBT1 = comboBox_BT1.SelectedIndex;
            int nBT2 = comboBox_BT2.SelectedIndex;

            int nResult = VTC6222_C4S.SetBTControl(nBT1, nBT2);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set BT Control success.";

                fnGetBTStatus();
            }
            else
            {
                textBox_Message.Text = "Set BT Control failed!!!";
            }
        }

        private void button_Set_GPS_Click(object sender, EventArgs e)
        {
            int nGPS_Enable = comboBox_GPS.SelectedIndex;
            int nResult = VTC6222_C4S.SetGPSControl(nGPS_Enable);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set GPS Control success.";

                fnGetGPSStatus();
            }
            else
            {
                textBox_Message.Text = "Set GPS Control failed!!!";
            }
        }

        private void button_Set_SIMcard_Click(object sender, EventArgs e)
        {
            int nSIMCardSelect1 = comboBox_SIM1.SelectedIndex;

            int nResult = VTC6222_C4S.SetSIMCardSelect(nSIMCardSelect1);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set SIM Card Select Success.";

                fnGetSIMCardStatus();
            }
            else
            {
                textBox_Message.Text = "Set SIM Card Select Failed!!!";
            }
        }
        #endregion

        #region Page 5 (G-Sensor)
        // ==============================================================================================================
        // Page 5 (G-Sensor)
        // ==============================================================================================================
        public void Page5Init()
        {
            comboBox_GSensorRegIndex.Items.Clear();
            comboBox_GSensorRegIndex.Items.Add("00 : DEVID");
            comboBox_GSensorRegIndex.Items.Add("29 : THRESH_TAP");
            comboBox_GSensorRegIndex.Items.Add("30 : OFSX");
            comboBox_GSensorRegIndex.Items.Add("31 : OFSY");
            comboBox_GSensorRegIndex.Items.Add("32 : OFSZ");
            comboBox_GSensorRegIndex.Items.Add("33 : DUR");
            comboBox_GSensorRegIndex.Items.Add("34 : Latent");
            comboBox_GSensorRegIndex.Items.Add("35 : Window");
            comboBox_GSensorRegIndex.Items.Add("36 : THRESH_ACT");
            comboBox_GSensorRegIndex.Items.Add("37 : THRESH_INACT");
            comboBox_GSensorRegIndex.Items.Add("38 : TIME_INACT");
            comboBox_GSensorRegIndex.Items.Add("39 : ACT_INACT_CTL");
            comboBox_GSensorRegIndex.Items.Add("40 : THRESH_FF");
            comboBox_GSensorRegIndex.Items.Add("41 : TIME_FF");
            comboBox_GSensorRegIndex.Items.Add("42 : TAP_AXES");
            comboBox_GSensorRegIndex.Items.Add("43 : ACT_TAP_STATUS");
            comboBox_GSensorRegIndex.Items.Add("44 : BW_RATE");
            comboBox_GSensorRegIndex.Items.Add("45 : POWER_CTL");
            comboBox_GSensorRegIndex.Items.Add("46 : INT_ENABLE");
            comboBox_GSensorRegIndex.Items.Add("47 : INT_MAP");
            comboBox_GSensorRegIndex.Items.Add("48 : INT_SOURCE");
            comboBox_GSensorRegIndex.Items.Add("49 : DATA_FORMAT");
            comboBox_GSensorRegIndex.Items.Add("50 : DATA_X0");
            comboBox_GSensorRegIndex.Items.Add("51 : DATA_X1");
            comboBox_GSensorRegIndex.Items.Add("52 : DATA_Y0");
            comboBox_GSensorRegIndex.Items.Add("53 : DATA_Y1");
            comboBox_GSensorRegIndex.Items.Add("54 : DATA_Z0");
            comboBox_GSensorRegIndex.Items.Add("55 : DATA_Z1");
            comboBox_GSensorRegIndex.Items.Add("56 : FIFO_CTL");
            comboBox_GSensorRegIndex.Items.Add("57 : FIFO_STATUS");
            comboBox_GSensorRegIndex.SelectedIndex = 17;

            textBox_WriteGSensor.Text = "0A";

            listView_GSensor.Clear();
            listView_GSensor.View = View.Details;
            listView_GSensor.LabelEdit = false;
            listView_GSensor.AllowColumnReorder = false;
            listView_GSensor.CheckBoxes = false;
            listView_GSensor.FullRowSelect = false;
            listView_GSensor.GridLines = true;
            listView_GSensor.Sorting = SortOrder.None;

            listView_GSensor.Columns.Clear();
            listView_GSensor.Columns.Add("Num", 40);
            listView_GSensor.Columns.Add("Name", 110);
            listView_GSensor.Columns.Add("Type", 40);
            listView_GSensor.Columns.Add("Value", 46);
            listView_GSensor.Columns.Add("Description", 300);
        }

        public byte GetAddress(int nIndex)
        {
            byte byAddress = 0;
            if (nIndex == 0)
                byAddress = 0;
            else
                byAddress = (byte)(nIndex + 0x1C);

            if (byAddress < 0 && byAddress > 0x39)
                byAddress = 0;

            return byAddress;
        }

        public string[] m_strName = new string[]{
						"DEVID",
						"Reserved",
						"THRESH_TAP",
						"OFSX",
						"OFSY",
						"OFSZ",
						"DUR",
						"Latent",
						"Window",
						"THRESH_ACT",
						"THRESH_INACT",
						"TIME_INACT",
						"ACT_INACT_CTL",
						"THRESH_FF",
						"TIME_FF",
						"TAP_AXES",
						"ACT_TAP_STATUS",
						"BW_RATE",
						"POWER_CTL",
						"INT_ENABLE",
						"INT_MAP",
						"INT_SOURCE",
						"DATA_FORMAT",
						"DATAX0",
						"DATAX1",
						"DATAY0",
						"DATAY1",
						"DATAZ0",
						"DATAZ1",
						"FIFO_CTL",
						"FIFO_STATUS"};

        public string[] m_strType = new string[]{
						"R",
						"",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R",
						"R/W",
						"R/W",
						"R/W",
						"R/W",
						"R",
						"R/W",
						"R",
						"R",
						"R",
						"R",
						"R",
						"R",
						"R/W",
						"R"};

        public string[] m_strDescription = new string[] {
						"Device ID",
						"Reserved; do not access",
						"Tap threshold",
						"X-axis offset",
						"Y-axis offset",
						"Z-axis offset",
						"Tap duration",
						"Tap latency",
						"Tap window",
						"Activity threshold",
						"Inactivity threshold",
						"Inactivity time",
						"Axis enable control for activity and inactivity detection",
						"Free-fall threshold",
						"Free-fall time",
						"Axis control for single tap/double tap",
						"Source of single tap/double tap",
						"Data rate and power mode control",
						"Power-saving features control",
						"Interrupt enable control",
						"Interrupt mapping control",
						"Source of interrupts",
						"Data format control",
						"X-Axis Data 0",
						"X-Axis Data 1",
						"Y-Axis Data 0",
						"Y-Axis Data 1",
						"Z-Axis Data 0",
						"Z-Axis Data 1",
						"FIFO control",
						"FIFO status"};

        public void fnSetGsensorData()
        {
            string strData = textBox_WriteGSensor.Text;
            if (textBox_WriteGSensor.Text.Length > 2)
            {
                strData = textBox_WriteGSensor.Text.Substring(textBox_WriteGSensor.Text.Length - 2, 2);
            }

            byte[] byDataArray = new byte[] { byte.Parse(strData, System.Globalization.NumberStyles.HexNumber) };
            byte byRegIdx = GetAddress(comboBox_GSensorRegIndex.SelectedIndex);

            int nResult = VTC6222_C4S.SetGsensorData(byRegIdx, byDataArray, 1);
            if (nResult == 1)
            {
                textBox_Message.Text = "Set Gsensor Data Success.";
            }
            else
            {
                textBox_Message.Text = "Set Gsensor Data Failed!!!";
            }
        }

        public void fnGetGsensorData()
        {
            int nRegIdx = (int)GetAddress(comboBox_GSensorRegIndex.SelectedIndex);
            byte[] byDataArray = new byte[] { 0 };

            int nResult = VTC6222_C4S.GetGsensorData(nRegIdx, byDataArray, 1);
            if (nResult == 1)
            {
                textBox_ReadGSensor.Text = byDataArray[0].ToString("X2");
            }
            else
            {
                textBox_ReadGSensor.Text = "?";
            }
        }

        public void fnGetAllGsensorData()
        {
            listView_GSensor.Items.Clear();

            byte[] byDataArray = new byte[128];
            for (int i = 0; i < 128; i++)
            {
                byDataArray[i] = 0;
            }

            int nResult = VTC6222_C4S.GetGsensorData(0, byDataArray, 1);
            if (nResult == 1)
            {
                string strGsensorData;
                strGsensorData = byDataArray[0].ToString("X2");

                ListViewItem item1 = new ListViewItem("0");
                item1.SubItems.Add(m_strName[0]);
                item1.SubItems.Add(m_strType[0]);
                item1.SubItems.Add(strGsensorData);
                item1.SubItems.Add(m_strDescription[0]);

                ListViewItem item2 = new ListViewItem("1~28");
                item2.SubItems.Add(m_strName[1]);
                item2.SubItems.Add(m_strType[1]);
                item2.SubItems.Add("");
                item2.SubItems.Add(m_strDescription[1]);

                listView_GSensor.Items.Add(item1);
                listView_GSensor.Items.Add(item2);

                for (int i = 0; i < 128; i++)
                {
                    byDataArray[i] = 0;
                }

                nResult = VTC6222_C4S.GetGsensorData(29, byDataArray, 29);
                if (nResult == 1)
                {
                    for (int i = 0; i < 29; i++)
                    {
                        string strData;
                        strData = byDataArray[i].ToString("X2");

                        ListViewItem item = new ListViewItem((i + 29).ToString());
                        item.SubItems.Add(m_strName[i + 2]);
                        item.SubItems.Add(m_strType[i + 2]);
                        item.SubItems.Add(strData);
                        item.SubItems.Add(m_strDescription[i + 2]);

                        listView_GSensor.Items.Add(item);
                    }

                    textBox_Message.Text = "Get all Gsensor Data Success.";

                    return;
                }
            }

            textBox_Message.Text = "Get all Gsensor Data Failed!!!";
        }

        public void fnGetGsensorDataXYZ()
        {
            byte[] byDataArray = new byte[7];
            for (int i = 0; i < 7; i++)
            {
                byDataArray[i] = 0;
            }

            int nResult = VTC6222_C4S.GetGsensorData(50, byDataArray, 6);
            if (nResult == 1)
            {
                textBox_GSensorX.Text = ((short)(byDataArray[0] | (byDataArray[1] << 8))).ToString();
                textBox_GSensorY.Text = ((short)(byDataArray[2] | (byDataArray[3] << 8))).ToString();
                textBox_GSensorZ.Text = ((short)(byDataArray[4] | (byDataArray[5] << 8))).ToString();
            }
        }

        private void button_ReadGSensor_Click(object sender, EventArgs e)
        {
            fnGetGsensorData();
        }

        private void button_WriteGSensor_Click(object sender, EventArgs e)
        {
            fnSetGsensorData();
        }

        private void button_GSensorRefresh_Click(object sender, EventArgs e)
        {
            fnGetAllGsensorData();
        }

        #endregion

        #region Page 6 (PoE)
        // ==============================================================================================================
        // Page 6 (PoE)
        // ==============================================================================================================
        public void Page6Init()
        {
        }

        void fnGetTotalConsumption()
        {
            int nTotalConsumption = 0;
            int nResult = VTC6222_C4S.GetTotalConsumption(ref nTotalConsumption);
            if( nResult == 1 )
            {
                textBox_TotalConsumption.Text = nTotalConsumption.ToString();
            }
            else
            {
                textBox_TotalConsumption.Text = "?";
            }
        }

        void fnGetPOEStatus()
        {
            byte byStatus = 0;
            int nResult = VTC6222_C4S.GetPOEStatus(ref byStatus);
            if( nResult == 1 )
            {
                if( (byStatus & 0x01) == 0x01 )
                    textBox_LanPortPW_1.Text = "Power on";
                else
                    textBox_LanPortPW_1.Text = "Power off";
                
                if( (byStatus & 0x02) == 0x02 )
                    textBox_LanPortPW_2.Text = "Power on";
                else
                    textBox_LanPortPW_2.Text = "Power off";
                
                if( (byStatus & 0x04) == 0x04 )
                    textBox_LanPortPW_3.Text = "Power on";
                else
                    textBox_LanPortPW_3.Text = "Power off";
                
                if( (byStatus & 0x08) == 0x08 )
                    textBox_LanPortPW_4.Text = "Power on";
                else
                    textBox_LanPortPW_4.Text = "Power off";
            }
            else
            {
                textBox_LanPortPW_1.Text = "?";
                textBox_LanPortPW_2.Text = "?";
                textBox_LanPortPW_3.Text = "?";
                textBox_LanPortPW_4.Text = "?";
            }
        }
        
        void fnGetPortPowerConsumption1()
        {
            int nConsumption = 0;
            int nResult = VTC6222_C4S.GetPortPowerConsumption(1, ref nConsumption);
            if( nResult == 1 )
            {
                textBox_PortPowerConsumption1.Text = nConsumption.ToString();
            }
            else
            {
                textBox_PortPowerConsumption1.Text = "?";
            }
        }

        void fnGetPortPowerConsumption2()
        {
            int nConsumption = 0;
            int nResult = VTC6222_C4S.GetPortPowerConsumption(2, ref nConsumption);
            if (nResult == 1)
            {
                textBox_PortPowerConsumption2.Text = nConsumption.ToString();
            }
            else
            {
                textBox_PortPowerConsumption2.Text = "?";
            }
        }

        void fnGetPortPowerConsumption3()
        {
            int nConsumption = 0;
            int nResult = VTC6222_C4S.GetPortPowerConsumption(3, ref nConsumption);
            if (nResult == 1)
            {
                textBox_PortPowerConsumption3.Text = nConsumption.ToString();
            }
            else
            {
                textBox_PortPowerConsumption3.Text = "?";
            }
        }

        void fnGetPortPowerConsumption4()
        {
            int nConsumption = 0;
            int nResult = VTC6222_C4S.GetPortPowerConsumption(4, ref nConsumption);
            if (nResult == 1)
            {
                textBox_PortPowerConsumption4.Text = nConsumption.ToString();
            }
            else
            {
                textBox_PortPowerConsumption4.Text = "?";
            }
        }
        
        void ShowLinkStatus(byte status, ref TextBox textbox)
        {
            byte byStatus1 = (byte)(status & 0x07);
            byte byStatus2 = (byte)((status & 0x70)>>4);
            
            string strString = "";
            switch(byStatus1)
            {
                case 0: strString = "Unknow"; break;
                case 1: strString = "Short"; break;
                case 2: strString = "Cpd too high"; break;
                case 3: strString = "RSIG too low"; break;
                case 4: strString = "Good"; break;
                case 5: strString = "RSIG too high"; break;
                case 6: strString = "Open Circuit"; break;
            }
            
            strString += ", ";
            
            switch(byStatus2)
            {
                case 0: strString += "Unknow"; break;
                case 1: strString += "Class 1"; break;
                case 2: strString += "Class 2"; break;
                case 3: strString += "Class 3"; break;
                case 4: strString += "Class 4"; break;
                case 6: strString += "Class 0"; break;
                case 7: strString += "Over-Current"; break;
            }

            textbox.Text = strString;
        }
        
        void fnGetAllPortLinkStatus()
        {
            VTC6222_C4S.PortLinkStatus LinkStatus;
            LinkStatus.status = new byte[4];
            for (int i = 0; i < 4; i++)
                LinkStatus.status[i] = 0x0;

            int nResult = VTC6222_C4S.GetAllPortLinkStatus(ref LinkStatus);
            if( nResult == 1 )
            {
                ShowLinkStatus(LinkStatus.status[0], ref textBox_AllPortLinkStatus_1);
                ShowLinkStatus(LinkStatus.status[1], ref textBox_AllPortLinkStatus_2);
                ShowLinkStatus(LinkStatus.status[2], ref textBox_AllPortLinkStatus_3);
                ShowLinkStatus(LinkStatus.status[3], ref textBox_AllPortLinkStatus_4);
            }
            else
            {
                textBox_AllPortLinkStatus_1.Text = "?";
                textBox_AllPortLinkStatus_2.Text = "?";
                textBox_AllPortLinkStatus_3.Text = "?";
                textBox_AllPortLinkStatus_4.Text = "?";
            }
        }

        void fnGetCHIPID()
        {
            int nChipID = 0;
            int nResult = VTC6222_C4S.GetCHIPID(ref nChipID);
            if( nResult == 1 )
            {
                textBox_ChipID.Text = nChipID.ToString("X");
            }
            else
            {
                textBox_ChipID.Text = "?";
            }
        }
        
        void fnGetPortLinkStatus()
        {
            byte byStatus = 0;
            int nResult = VTC6222_C4S.GetPortLinkStatus(ref byStatus);
            if( nResult == 1 )
            {
                if( (byStatus & 0x01) == 0x01 )
                    textBox_PortLinkStatus_1.Text = "Link";
                else
                    textBox_PortLinkStatus_1.Text = "No Link";
                
                if( (byStatus & 0x02) == 0x02 )
                    textBox_PortLinkStatus_2.Text = "Link";
                else
                    textBox_PortLinkStatus_2.Text = "No Link";
                
                if( (byStatus & 0x04) == 0x04 )
                    textBox_PortLinkStatus_3.Text = "Link";
                else
                    textBox_PortLinkStatus_3.Text = "No Link";
                
                if( (byStatus & 0x08) == 0x08 )
                    textBox_PortLinkStatus_4.Text = "Link";
                else
                    textBox_PortLinkStatus_4.Text = "No Link";
            }
            else
            {
                textBox_PortLinkStatus_1.Text = "?";
                textBox_PortLinkStatus_2.Text = "?";
                textBox_PortLinkStatus_3.Text = "?";
                textBox_PortLinkStatus_4.Text = "?";
            }
        }
        
        void fnGetPortLinkSpeed()
        {
            VTC6222_C4S.PortLinkSpeed LinkSpeed;
            LinkSpeed.speed = new byte[4];
            for (int i = 0; i < 4; i++)
                LinkSpeed.speed[i] = 0x0;

            int nResult = VTC6222_C4S.GetPortLinkSpeed(ref LinkSpeed);
            if( nResult == 1 )
            {
                switch(LinkSpeed.speed[0])
                {
                    case 0: textBox_LinkSpeed_1.Text = "No"; break;
                    case 1: textBox_LinkSpeed_1.Text = "10 Mbps"; break;
                    case 2: textBox_LinkSpeed_1.Text = "100 Mbps"; break;
                    case 3: textBox_LinkSpeed_1.Text = "1000 Mbps"; break;
                }
                
                switch(LinkSpeed.speed[1])
                {
                    case 0: textBox_LinkSpeed_2.Text = "No"; break;
                    case 1: textBox_LinkSpeed_2.Text = "10 Mbps"; break;
                    case 2: textBox_LinkSpeed_2.Text = "100 Mbps"; break;
                    case 3: textBox_LinkSpeed_2.Text = "1000 Mbps"; break;
                }
                
                switch(LinkSpeed.speed[2])
                {
                    case 0: textBox_LinkSpeed_3.Text = "No"; break;
                    case 1: textBox_LinkSpeed_3.Text = "10 Mbps"; break;
                    case 2: textBox_LinkSpeed_3.Text = "100 Mbps"; break;
                    case 3: textBox_LinkSpeed_3.Text = "1000 Mbps"; break;
                }
                
                switch(LinkSpeed.speed[3])
                {
                    case 0: textBox_LinkSpeed_4.Text = "No"; break;
                    case 1: textBox_LinkSpeed_4.Text = "10 Mbps"; break;
                    case 2: textBox_LinkSpeed_4.Text = "100 Mbps"; break;
                    case 3: textBox_LinkSpeed_4.Text = "1000 Mbps"; break;
                }
            }
            else
            {
                textBox_LinkSpeed_1.Text = "?";
                textBox_LinkSpeed_2.Text = "?";
                textBox_LinkSpeed_3.Text = "?";
                textBox_LinkSpeed_4.Text = "?";
            }
        }

        private void button_PortPowerConsumption_Click(object sender, EventArgs e)
        {
            fnGetPortPowerConsumption1();
            fnGetPortPowerConsumption2();
            fnGetPortPowerConsumption3();
            fnGetPortPowerConsumption4();
        }

        private void button_AllPortLinkStatus_Click(object sender, EventArgs e)
        {
            fnGetAllPortLinkStatus();
        }

        private void button_PortLinkStatus_Click(object sender, EventArgs e)
        {
            fnGetPortLinkStatus();
        }

        private void button_LinkSpeed_Click(object sender, EventArgs e)
        {
            fnGetPortLinkSpeed();
        }

        private void button_LanPortPW_Click(object sender, EventArgs e)
        {
            fnGetPOEStatus();
        }

        private void button_LanPortPW_On_1_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(1, 1);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_1.Text = "?";
            }
        }

        private void button_LanPortPW_On_2_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(2, 1);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_2.Text = "?";
            }
        }

        private void button_LanPortPW_On_3_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(3, 1);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_3.Text = "?";
            }
        }

        private void button_LanPortPW_On_4_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(4, 1);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_4.Text = "?";
            }
        }

        private void button_LanPortPW_Off_1_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(1, 0);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_1.Text = "?";
            }
        }

        private void button_LanPortPW_Off_2_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(2, 0);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_2.Text = "?";
            }
        }

        private void button_LanPortPW_Off_3_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(3, 0);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_3.Text = "?";
            }
        }

        private void button_LanPortPW_Off_4_Click(object sender, EventArgs e)
        {
            int nResult = VTC6222_C4S.SetPortPowerControl(4, 0);
            if (nResult == 1)
            {
            }
            else
            {
                textBox_LanPortPW_4.Text = "?";
            }
        }
        #endregion

        #region Page 7 (Event)
        // ==============================================================================================================
        // Page 7 (Event)
        // ==============================================================================================================
        public void Page7Init()
        {
            fnCallBackFun = new VTC6222_C4S.EventSystemAlarm_CallBack(EventSystemAlarm);
            VTC6222_C4S.RegisterSystemAlarmCallBackFunction(fnCallBackFun);

            listBox_Event.Sorted = false;

            label_Event.Text = "System event type : \r1. Over voltage\r2. Lower voltage\r3. Over temperature\r4. Lower temperature";
        }

        VTC6222_C4S.EventSystemAlarm_CallBack fnCallBackFun = null;

        public void EventSystemAlarm(byte byAlarm)
        {
            DateTime dtTime = DateTime.Now;

            string strTime;
            strTime = dtTime.Year.ToString("D4") + "/" + dtTime.Month.ToString("D2") + "/" + dtTime.Day.ToString("D2") + " " + dtTime.Hour.ToString("D2") + ":" + dtTime.Minute.ToString("D2") + ":" + dtTime.Second.ToString("D2") + ":" + dtTime.Millisecond.ToString("D3") + " ==> ";

            if ((byAlarm & 0x01) == 0x01)
            {
                Add_Message(strTime + "Over voltage");
            }

            if ((byAlarm & 0x02) == 0x02)
            {
                Add_Message(strTime + "Lower voltage");
            }

            if ((byAlarm & 0x04) == 0x04)
            {
                Add_Message(strTime + "Over temperature");
            }

            if ((byAlarm & 0x08) == 0x08)
            {
                Add_Message(strTime + "Lower temperature");
            }
        }


        delegate void Add_Message_Callback(string msg);
        private void Add_Message(string msg)
        {
            if (this.listBox_Event.InvokeRequired)
            {
                Add_Message_Callback d = new Add_Message_Callback(Add_Message);
                this.Invoke(d, new object[] { msg });
            }
            else
            {
                while (listBox_Event.Items.Count >= 200)
                {
                    listBox_Event.Items.RemoveAt(0);
                }

                listBox_Event.Items.Add(msg);

                listBox_Event.SelectedIndex = listBox_Event.Items.Count - 1;
            }
        }
        #endregion

        #region Page Selecting
        // ==============================================================================================================
        // Page Selecting
        // ==============================================================================================================
        public void PageSelectingInit()
        {
            timer_Page1.Interval = 100;
            timer_Page1.Start();
        }

        public void ReadingAfterLoadDefault()
        {
            m_nPage1_Step = 0;
            m_nPage2_Step = 0;
            m_nPage3_Step = 0;
            m_nPage4_Step = 0;

            tabControl1_Selecting(this, null);
        }

        private void tabControl1_Selecting(object sender, TabControlCancelEventArgs e)
        {
            textBox_Message.Text = "";

            switch (tabControl1.SelectedIndex)
            {
                case 0:
                    {
                        timer_Page1.Interval = 10;
                        timer_Page1.Start();
                    }
                    break;
                case 1:
                    {
                        timer_Page2.Interval = 10;
                        timer_Page2.Start();
                    }
                    break;
                case 2:
                    {
                        timer_Page3.Interval = 10;
                        timer_Page3.Start();
                    }
                    break;
                case 3:
                    {
                        timer_Page4.Interval = 10;
                        timer_Page4.Start();
                    }
                    break;
                case 4:
                    {
                        timer_Page5.Interval = 10;
                        timer_Page5.Start();
                    }
                    break;
                case 5:
                    {
                        timer_Page6.Interval = 10;
                        timer_Page6.Start();
                    }
                    break;
            }
        }

        private uint m_nPage1_Step = 0;
        private void timer_Page1_Tick(object sender, EventArgs e)
        {
            timer_Page1.Stop();

            switch (m_nPage1_Step)
            {
                case 0: fnGetMCUVersion(); m_nPage1_Step++; break;
                case 1: fnGetIgnitionStatus(); m_nPage1_Step++; break;
                case 2: fnGetSupplyVoltage(); m_nPage1_Step++; break;
                case 3: fnGetPowerType(); m_nPage1_Step++; break;
                case 4: fnGetStartupShutdownOption(); m_nPage1_Step++; break;
                default: break;
            }

            if (m_nPage1_Step < 5)
            {
                timer_Page1.Interval = 100;
                timer_Page1.Start();
            }
            else
            {
                timer_Page1_AutoRead.Interval = 100;
                timer_Page1_AutoRead.Start();
            }
        }

        private uint m_nPage1_ReadCount = 0;
        private void timer_Page1_AutoRead_Tick(object sender, EventArgs e)
        {
            timer_Page1_AutoRead.Stop();

            if (tabControl1.SelectedIndex == 0)
            {
                switch (m_nPage1_ReadCount % 2)
                {
                    case 0: fnGetIgnitionStatus(); break;
                    case 1: fnGetSupplyVoltage(); break;
                }

                m_nPage1_ReadCount++;
            }

            timer_Page1_AutoRead.Interval = 200;
            timer_Page1_AutoRead.Start();
        }

        private uint m_nPage2_Step = 0;
        private void timer_Page2_Tick(object sender, EventArgs e)
        {
            timer_Page2.Stop();

            switch (m_nPage2_Step)
            {
                case 0: fnGetDelayTimeOption(); m_nPage2_Step++; break;
                case 1: fnGetRTCStatus(); m_nPage2_Step++; break;
                case 2: fnGetAlarmTimer(); m_nPage2_Step++; break;
                case 3: fnGetWatchdogConfig(); m_nPage2_Step++; break;
                default: break;
            }

            if (m_nPage2_Step < 4)
            {
                timer_Page2.Interval = 50;
                timer_Page2.Start();
            }
        }

        private uint m_nPage3_Step = 0;
        private void timer_Page3_Tick(object sender, EventArgs e)
        {
            timer_Page3.Stop();

            switch (m_nPage3_Step)
            {
                case 0: fnGetGPOStatus(); m_nPage3_Step++; break;
                case 1: fnGetGPIStatus(); m_nPage3_Step++; break;
                case 2: fnGetWakeOnLanStatus(); m_nPage3_Step++; break;
                case 3: fnGetExternal12VStatus(); m_nPage3_Step++; break;
                case 4: fnGetProgramLEDStatus(); m_nPage3_Step++; break;
                default: break;
            }

            if (m_nPage3_Step < 5)
            {
                timer_Page3.Interval = 100;
                timer_Page3.Start();
            }
            else
            {
                timer_Page3_AutoRead.Interval = 100;
                timer_Page3_AutoRead.Start();
            }
        }

        private void timer_Page3_AutoRead_Tick(object sender, EventArgs e)
        {
            timer_Page3_AutoRead.Stop();

            if (tabControl1.SelectedIndex == 2)
            {
                fnGetGPIStatus();
            }

            timer_Page3_AutoRead.Interval = 500;
            timer_Page3_AutoRead.Start();
        }

        private uint m_nPage4_Step = 0;
        private void timer_Page4_Tick(object sender, EventArgs e)
        {
            timer_Page4.Stop();

            switch (m_nPage4_Step)
            {
                case 0: fnGetWWANStatus(); m_nPage4_Step++; break;
                case 1: fnGetWiFiStatus(); m_nPage4_Step++; break;
                case 2: fnGetBTStatus(); m_nPage4_Step++; break;
                case 3: fnGetGPSStatus(); m_nPage4_Step++; break;
                case 4: fnGetSIMCardStatus(); m_nPage4_Step++; break;
                default: break;
            }

            if (m_nPage4_Step < 5)
            {
                timer_Page4.Interval = 100;
                timer_Page4.Start();
            }
        }

        private uint m_nPage5_Step = 0;
        private void timer_Page5_Tick(object sender, EventArgs e)
        {
            timer_Page5.Stop();

            switch (m_nPage5_Step)
            {
                case 0: fnSetGsensorData(); m_nPage5_Step++; break;
                case 1: fnGetGsensorData(); m_nPage5_Step++; break;
                case 2: fnGetAllGsensorData(); m_nPage5_Step++; break;
                default: break;
            }

            if (m_nPage5_Step < 3)
            {
                timer_Page5.Interval = 100;
                timer_Page5.Start();
            }
            else
            {
                timer_Page5_AutoRead.Interval = 200;
                timer_Page5_AutoRead.Start();
            }
        }

        private void timer_Page5_AutoRead_Tick(object sender, EventArgs e)
        {
            timer_Page5_AutoRead.Stop();

            if (tabControl1.SelectedIndex == 4)
            {
                fnGetGsensorDataXYZ();
            }

            timer_Page5_AutoRead.Interval = 200;
            timer_Page5_AutoRead.Start();
        }

        private uint m_nPage6_Step = 0;
        private void timer_Page6_Tick(object sender, EventArgs e)
        {
            timer_Page6.Stop();

            switch (m_nPage6_Step)
            {
                case 0: fnGetPOEStatus(); m_nPage6_Step++; break;
                case 1: fnGetPortPowerConsumption1(); m_nPage6_Step++; break;
                case 2: fnGetPortPowerConsumption2(); m_nPage6_Step++; break;
                case 3: fnGetPortPowerConsumption3(); m_nPage6_Step++; break;
                case 4: fnGetPortPowerConsumption4(); m_nPage6_Step++; break;
                case 5: fnGetAllPortLinkStatus(); m_nPage6_Step++; break;
                case 6: fnGetPortLinkStatus(); m_nPage6_Step++; break;
                case 7: fnGetPortLinkSpeed(); m_nPage6_Step++; break;
                case 8: fnGetTotalConsumption(); m_nPage6_Step++; break;
                case 9: fnGetCHIPID(); m_nPage6_Step++; break;
                default: break;
            }

            if (m_nPage6_Step < 10)
            {
                timer_Page6.Interval = 10;
                timer_Page6.Start();
            }
            else
            {
                timer_Page6_AutoRead.Interval = 1000;
                timer_Page6_AutoRead.Start();
            }
        }

        private uint m_nPage6_ReadCount = 0;
        private void timer_Page6_AutoRead_Tick(object sender, EventArgs e)
        {
            timer_Page6_AutoRead.Stop();

            if (tabControl1.SelectedIndex == 5)
            {
                switch (m_nPage6_ReadCount % 6)
                {
                    case 0: fnGetTotalConsumption();    break;
                    case 1: fnGetPOEStatus();  break;
                    case 2:
                        fnGetPortPowerConsumption1();
                        fnGetPortPowerConsumption2();
                        fnGetPortPowerConsumption3();
                        fnGetPortPowerConsumption4();
                        break;
                    case 3: fnGetAllPortLinkStatus();  break;
                    case 4: fnGetPortLinkStatus();  break;
                    case 5: fnGetPortLinkSpeed();  break;
                }

                m_nPage6_ReadCount++;
            }

            timer_Page6_AutoRead.Interval = 200;
            timer_Page6_AutoRead.Start();
        }
        #endregion
    }
}
